# NB_ExternalFile has been derived from the following products:
#
#- ExtFile
#Copyright (c) 2001 Gregor Heine <mac.gregor@gmx.de>. All rights reserved.
#ExtFile Home: http://www.zope.org/Members/MacGregor/ExtFile/index_html

#- ExternalFile
#Copyright (c) 2002 Ariel Partners LLC, all right reserved.
#

from Globals import InitializeClass
import OFS
from OFS.SimpleItem import SimpleItem
from OFS.ObjectManager import ObjectManager
from AccessControl import ClassSecurityInfo
from cStringIO import StringIO
import os, os.path, string, types, urllib, mimetypes, sys
from NB_Utils import getDTML, copyFile, getReqAttr

manage_addExternalFileForm=getDTML('dtml/Class/ExternalFileAdd')
def manage_addExternalFile(self,
			id,
			title='',
			description='',
			file_path='',
			hard_link = 0,
			ext_file = '',
			REQUEST=None):
	"""
		Add External File
	"""

	if ext_file.filename:
		copyFile(ext_file, file_path)
	
	instance = NB_ExternalFile(id, title, description, file_path, hard_link)
	self._setObject(id, instance)
	
	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)
	

class NB_ExternalFile(SimpleItem):

	"""
	NeoBoard ExternalFile Class
	"""
	
	meta_type = 'NeoBoard External File'
	icon='misc_/NeoBoard/external_file_icon'
	
	_default_mime_type = 'application/octet-stream'
	_default_wrapper = 'File'
	
	_wrappers_list = [ 'File', 'Image'  ]
	
	manage_options = (
		{'label':'Edit', 'action': 'manage_editForm'},
		{'label':'View', 'action': 'manage_view'},
		{'label':'Security',   'action':'manage_access'},
		{'label':'Ownership',  'action':'manage_owner'},
		{'label':'Undo',       'action':'manage_UndoForm'},
	)	
	
	security = ClassSecurityInfo()
	
	security.declareObjectProtected('View')	
        
	def __init__(self, id, title, description, file_path, hard_link=0):
		self.id          = id       
		self.title       = title
		self.description = description
		self.file_path    = file_path
		self.hard_link = hard_link
		
		mime_type, encoding = mimetypes.guess_type(file_path)
		
		if mime_type:
			self.content_type = mime_type
		else:
			self.content_type = self._getDefaultMimeType()

        	type, subtype  = self.content_type.split('/', 2)
        
	        if type == 'image':
	            self.setWrapper('Image')
	        elif type == 'text':
	            self.setWrapper(self.getDefaultWrapper())
	        else:
	            self.setWrapper('File')
            	
	def getContentType(self):
		return self.content_type
		
	def getFilePath(self):
		return self.file_path
	
	def getFileName(self):
		return os.path.basename(self.file_path)
	
	def getFileSize(self):
		try:
			filesize = os.stat(self.getFilePath())[6]
		except:
			filesize = 0
		return filesize

	def getFileSizeInKB(self):
		return round(int(self.getFileSize()/1024))
	
	def getFileModTime(self):
		return os.path.getmtime(self.getFilePath())

	def getModTime(self):
		file_mtime = self.getFileModTime()
		if self._p_mtime > file_mtime:
			return self._p_mtime
		else:
			return file_mtime
		
	def getRawContent(self):
		data = StringIO()
		
		copyFile(self.getFilePath(), data)

		return data.getvalue()
	
	def setRawContent(self, content):
		data = StringIO(content)
		copyFile(data, self.getFilePath())
	
	def index_html(self, REQUEST=None):
		"""
			Own index_html
		"""
		
		return self.__call__(None, REQUEST)
	
	def __str__(self, REQUEST=None):
		return self.__call__(None, REQUEST)
			
	def __len__(self): 
		return 1		
	
	def __call__(self, client=None, REQUEST=None):
		"""
			Own __call__
		"""	

		wrapper = self.getWrapper()
		data = self.getRawContent()
		
		if wrapper == 'Image':
			resultImage = OFS.Image.Image(self.id, 
						self.title,
			                        data,
			                        self.getContentType())
			resultImage._p_mtime     = self.getModTime()
			resultImage.absolute_url = self.absolute_url
			resultImage.content_type = self.getContentType()

			if REQUEST is not None:			
				resp = REQUEST['RESPONSE'] 
				return resultImage.index_html(REQUEST, resp)
			else:
				return resultImage.__str__()

		elif wrapper == 'File':
			resultFile = OFS.Image.File(self.id, 
						self.title,
			                        data,
			                        self.getContentType())

			resultFile._p_mtime     = self.getModTime()
			resultFile.absolute_url = self.absolute_url
			resultFile.content_type = self.getContentType()
			if REQUEST is not None:
				resp = REQUEST['RESPONSE']
				resp.setHeader('Content-Disposition',' inline; filename=' + self.getFileName())
				return resultFile.index_html(REQUEST, resp)
			else:
				return resultFile.__str__()
	
	def __len__(self): 
		return 1
		
	security.declareProtected('Edit NeoBoard External File', 'manage_editExternalFile')
	def manage_editExternalFile(self, 
			title = '',
			description = '',
			wrapper = '',
			ext_file = '',
			file_path = '',
			hard_link=0,
			REQUEST=None):
		"""
			Edit External File
		"""
		
		self.title = title
		self.description = description			
		self.hard_link = hard_link
		if wrapper:
			self.setWrapper(wrapper)
		if not file_path == self.getFilePath():
			copyFile(self.getFilePath(), file_path)
			if self.isHardLink() or hard_link:
				try:
					os.unlink(self.getFilePath())
					self._delThumbnail()
				except:
					pass
					
			self.file_path = file_path

			mime_type, encoding = mimetypes.guess_type(file_path)
		
			if mime_type:
				self.content_type = mime_type
			else:
				self.content_type = self._getDefaultMimeType()

	        	type, subtype  = self.content_type.split('/', 2)
	        
		        if type == 'image':
		            self.setWrapper('Image')
		        elif type == 'text':
		            self.setWrapper(self.getDefaultWrapper())
		        else:
		            self.setWrapper('File')
			
		if ext_file.filename:
			self.setRawContent(ext_file.read())
			
		if REQUEST is not None:
			return self.manage_editForm(self, REQUEST)


	security.declareProtected('Edit NeoBoard External File', 'manage_deleteExternalFile')
	def manage_deleteExternalFile(self, REQUEST=None):
		"""
			Delete External File
		"""
				
		self.aq_inner.aq_parent._delObject(self.id)
		
		if REQUEST is not None:
			return self.manage_editExternalFile(title, description, wrapper, content, hard_link, REQUEST)
	
	def manage_beforeDelete(self, item, container):
			if self.isHardLink():				
				try:
					os.unlink(self.getFilePath())			
				except:
					pass
					
			self._delThumbnail()						
	
	def getWrappersList(self):
		return self._wrappers_list

	def setWrapper(self, wrapper):
		self._wrapper = wrapper
	
	def getWrapper(self):
		if self._wrapper in self.getWrappersList():
			return self._wrapper
		else:
			self.setWrapper(self.getDefaultWrapper())
			return self._wrapper

	def getDefaultWrapper(self):
		return self._default_wrapper

	def isHardLink(self):
		return self.hard_link

	def thumb(self, REQUEST=None):
		"""
			Create and Display Thumbnail of this file
		"""
		
		if not self.getWrapper() == 'Image':
			return ''
	
		if REQUEST is not None:
			max_width = getReqAttr(REQUEST, 'max_width')		
			max_height = getReqAttr(REQUEST, 'max_height')		
			keep_ratio = getReqAttr(REQUEST, 'keep_ratio')		
			force = getReqAttr(REQUEST, 'force')
		
		if not max_width:
			max_width = 10
		if not max_height:
			max_height = 10
		
		if keep_ratio is None:
			keep_ratio = 1
		
		if force is None:
			force = 0
		
		try:
			thumbnail_file = self._createThumbnail(image_file=self.getFilePath(), max_width=int(max_width), max_height=int(max_height), keep_ratio=int(keep_ratio), force=int(force))		
			resultThumbnail = OFS.Image.Image(self.id, 
						self.title,
			                        open(thumbnail_file, 'rb').read(),
			                        self.getContentType())
			resultThumbnail._p_mtime     = self.getModTime()
			resultThumbnail.absolute_url = self.absolute_url
			resultThumbnail.content_type = 'image/jpeg'
	
			if REQUEST is not None:			
				resp = REQUEST['RESPONSE'] 
				return resultThumbnail.index_html(REQUEST, resp)
			else:
				return resultThumbnail.__str__()		
		except:
			return ''			


	def getImageSize(self, image_file=None):
		if image_file is None:	
			if not self.getWrapper() == 'Image':
				return 0,0
			if hasattr(self, 'image_width') and hasattr(self, 'image_height'):
				if self.image_width > 0 and self.image_height > 0:
					return self.image_width, self.image_height
			
			try:
				from PIL import Image
	
				im = Image.open(self.getFilePath()) 
				self.image_width = im.size[0]
				self.image_height = im.size[1]
			except:
				self.image_width = self.image_height = 0
			return self.image_width, self.image_height
		
		else:
			try:
				im = Image.open(image_file) 
				image_width = im.size[0]
				image_height = im.size[1]
			except:
				image_width = image_height = 0
			
			return image_width, image_height
			
	security.declarePrivate('_getDefaultMimeType')
	def _getDefaultMimeType(self):
		return self._default_mime_type

	security.declarePrivate('_createThumbnail')
	def _createThumbnail(self, image_file, max_width, max_height, keep_ratio=0, force=0):
	
		save_file_name = os.path.splitext(image_file)[0] + '_thumb' + '.jpg'
		width, height = self.getImageSize()
		if width <= max_width and height <= max_height:
			return image_file
		
		if os.path.isfile(save_file_name) and not force:
			width, height = self.getImageSize(save_file_name)
			if width == max_width and height == max_height:
				return save_file_name
	
		from PIL import Image
		im = Image.open(image_file) 
		if im.mode!='RGB' and im.mode!='CMYK': 
			im = im.convert("RGB")
		
		if keep_ratio==1:
			im.thumbnail((max_width,max_height))
		else: 					
			im.draft(None,(max_width,max_height))
			im.load()
			im = im.resize((max_width,max_height), Image.BICUBIC)
		
		im.save(save_file_name,'JPEG', quality = 100)
		
		return save_file_name

	security.declarePrivate('_delThumbnail')
	def _delThumbnail(self):
		thumnail_file = os.path.splitext(self.getFilePath())[0] + '_thumb' + '.jpg'
		try:
			os.unlink(thumnail_file)
		except:
			pass


	security.declareProtected('Edit NeoBoard External File', 'selfUpgrade')
	def selfUpgrade(self):
		pass
	
InitializeClass(NB_ExternalFile)
