# NB_Mailer class supports both plain text/html mails

import sys, smtplib, MimeWriter, base64, StringIO

class NB_Mailer:

	def __init__(self, from_addr, to_addr, reply_addr, subject, body, attachments=None, html=0, smtp_host='localhost', smtp_port=25):
		self.smtp_host = smtp_host
		self.smtp_port = int(smtp_port)
		self.from_addr = from_addr
		self.to_addr = to_addr
		self.reply_addr = reply_addr
		self.subject = subject
		self.body = body
		self.attachments = attachments
		self.is_html = html
		self.cc_addr = self.bcc_addr = None

	def setCC(self, cc_addr):
		self.cc_addr = cc_addr

	def setBCC(self, bcc_addr):
		self.bcc_addr = bcc_addr
		
	def send(self):
		
		message = StringIO.StringIO()
		writer = MimeWriter.MimeWriter(message)
		writer.addheader('MIME-Version', '1.0')
		writer.addheader('From', self.from_addr)
		writer.addheader('To', self.to_addr)
		writer.addheader('Reply-to', self.reply_addr)
		
		if self.cc_addr is not None:
			writer.addheader('CC', self.cc_addr)

		if self.bcc_addr is not None:
			writer.addheader('BCC', self.bcc_addr)
		
		writer.addheader('Subject', self.subject)
		writer.startmultipartbody('mixed')
	
		part = writer.nextpart()
		
		if self.is_html:
			body = part.startbody('text/html')
		else:
			body = part.startbody('text/plain')		
		
		body.write(self.body)
	
		if self.attachments is not None:
			for file in self.attachments:
				part = writer.nextpart()
				part.addheader('Content-Transfer-Encoding', 'base64')
				body = part.startbody(file['content_type'])
				base64.encode(file['content'], body)
	
		writer.lastpart()
	
		smtp = smtplib.SMTP(host=self.smtp_host, port=self.smtp_port)
		smtp.sendmail(self.from_addr, self.to_addr, message.getvalue())
		smtp.quit()
		

