# NeoBoard

from Globals import InitializeClass, MessageDialog, package_home
from AccessControl import ClassSecurityInfo
from string import replace, join, split, count
from NB_Utils import getDTML, addImage, traverseFolder, getNextElementName, setFormMsg, setFormError
from NB_BaseFolder import NB_BaseFolder
from NB_Mailer import NB_Mailer
import os

stylesheet = getDTML('dtml/stylesheet')

manage_addNeoBoardForm =  getDTML('dtml/NeoBoardAdd')
def manage_addNeoBoard(self, 
			id=None, 
			title='', 
			description='', 
			archive_path='',
			max_file_size=5,
			max_image_width=200,
			max_image_height=200,
			max_list_title_length=30,
			enabled=0,
			read_only=0,
			page_limit=20,
			posting_categories=[],			
			managers=[],
			mail_article=0,
			REQUEST=None):
			
	""" Add NeoBoard"""		                            

	if id is None:
		return MessageDialog(title='Invalid ID!',
	                             message='ID required!',
	                             action=REQUEST['URL1']+'/manage_main'
	                            )

	instance = NeoBoard()
	
	instance.id=str(id)
	instance.title=instance.menu_title=str(title)
	
	instance.description=str(description)

	if not str(archive_path).endswith('/'):
		archive_path = archive_path + '/'


	if not os.path.isdir(archive_path):
		try:
			os.mkdir(archive_path, 0755)
		except:
			return MessageDialog(title='Invalid Archive Path!',
		                             message="Archive path doesn't exist and cannot be created!",
		                             action=REQUEST['URL1']+'/manage_main'
		                            )

	instance.archive_path=str(archive_path)

	instance.max_file_size=max_file_size
	instance.max_image_width=max_image_width
	instance.max_image_height=max_image_height
	instance.max_list_title_length=max_list_title_length

	instance.page_limit = page_limit

	instance.enabled=enabled
	instance.read_only=read_only

	instance.editManagers(managers)
	instance.editPostingCategories(posting_categories)

	instance.mail_article=mail_article
		
	instance.manage_addFolder('icons','NeoBoard Icons')

	instance.addIcon('icon_filler', 'Filler')

	instance.addIcon('icon_thread', 'Thread')
	instance.addIcon('icon_reply', 'Thead Reply')
	instance.addIcon('icon_reading', 'Reading')
	instance.addIcon('icon_new', 'New')
	
	instance.addIcon('button_list', 'List')
	instance.addIcon('button_write', 'Write')
	instance.addIcon('button_modify', 'Modify')
	instance.addIcon('button_delete', 'Delete')
	instance.addIcon('button_reply', 'Reply')
	instance.addIcon('button_move', 'Move')
	instance.addIcon('button_vote', 'Vote')
	instance.addIcon('button_collapse_thread', 'Collapse Thread')
	instance.addIcon('button_expand_thread', 'Expand Thread')
	
	instance.addIcon('mime_zip', 'Zip File')
	instance.addIcon('mime_audio', 'Audio File')
	instance.addIcon('mime_video', 'Video File')
	instance.addIcon('mime_text', 'Text File')
	instance.addIcon('mime_html', 'HTML File')
	instance.addIcon('mime_docs', 'Document File')
	instance.addIcon('mime_exe', 'Exe File')
	instance.addIcon('mime_img', 'Image File')
	instance.addIcon('mime_unknown', 'Unknown File')

	instance.setNextArticleNum(0)
	instance.setTotalNumArticles(0)
	
	self._setObject(id, instance)
	
	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)	

class NeoBoard(NB_BaseFolder):
	"""
		NeoBoard Class
	"""
	meta_type='NeoBoard'
	icon='misc_/NeoBoard/neoboard_icon'     

	meta_types=(
		{'name': 'NeoBoard Article', 'action':'addNeoBoardArticleForm', 'permission': 'Add NeoBoard Article'},
	)

	all_meta_types = meta_types

	manage_options=(
		{'label':'Conents',    'action':'manage_main'},
		{'label':'View',       'action':'index_html'},
		{'label':'Properties', 'action':'manage_editForm'},
		{'label':'Security',   'action':'manage_access'},
		{'label':'Ownership',  'action':'manage_owner'},
	)

	security = ClassSecurityInfo()	
	security.declareObjectProtected('View')	
	
	NB_PAGE_NAV = getDTML('dtml/PageNav')
	
	neoboard_header = getDTML('dtml/NeoBoardHeader')
	neoboard_footer = getDTML('dtml/NeoBoardFooter')
	neoboard_menu = getDTML('dtml/NeoBoardMenu')
	article_list = getDTML('dtml/NeoBoardArticleList')
	
	mail_html_header = getDTML('dtml/mail_html_header')
	mail_html_footer = getDTML('dtml/mail_html_footer')
	
	post = getDTML('dtml/NeoBoardArticleAdd')
	

	security.declareProtected('Edit NeoBoard', 'manage_editForm')
	manage_editForm = getDTML('dtml/NeoBoardEdit')
	

	def __init__(self):
		pass

	security.declareProtected('Edit NeoBoard', 'manage_editNeoBoard')
	def manage_editNeoBoard(self, 
			title='', 
			description='', 
			archive_path='',
			max_file_size=5,
			max_image_width=200,
			max_image_height=200,
			max_list_title_length=30,
			page_limit=20,
			enabled=0,
			read_only=0,
			posting_categories=[],			
			managers=[],
			icon_thread='',
			icon_reply='',
			icon_reading='',
			icon_new='',
			button_list='',
			button_write='',
			button_modify='',
			button_delete='',
			button_reply='',
			button_move='',
			button_vote='',
			button_collapse_thread='',
			button_expand_thread='',
	                mime_zip='',
	                mime_audio='',
	                mime_video='',
	                mime_text='',
	                mime_html='',
	                mime_docs='',
	                mime_exe='',
	                mime_img='',
	                mime_unknown='',
			mail_article=0,
			REQUEST=None):
		""" Edit NeoBoard"""		                            

		self.title=self.menu_title=str(title)
		
		self.description=str(description)
		
		if not str(archive_path).endswith('/'):
			archive_path = archive_path + '/'
			
		if not os.path.isdir(archive_path):
			try:
				os.mkdir(archive_path, 0755)
			except:
				return MessageDialog(title='Invalid Archive Path!',
			                             message="Archive path doesn't exist and cannot be created!",
			                             action=REQUEST['URL1']+'/manage_main'
			                            )
			
		self.archive_path=str(archive_path)
		
		self.page_limit = page_limit
		self.enabled=enabled
		self.read_only=read_only
			
		self.editManagers(managers)
		self.editPostingCategories(posting_categories)

		self.max_file_size = max_file_size
		self.max_image_width=max_image_width
		self.max_image_height=max_image_height
		self.max_list_title_length=max_list_title_length
		self.mail_article=mail_article
				
		self.editIcon('icon_thread', 'Thread',  icon_thread)
		self.editIcon('icon_reply', 'Thead Reply',  icon_reply)
		self.editIcon('icon_reading', 'Reading',  icon_reading)
		self.editIcon('icon_new', 'New',  icon_new)
		
		self.editIcon('button_list', 'List',  button_list)
		self.editIcon('button_write', 'Write',  button_write)
		self.editIcon('button_modify', 'Modify',  button_modify)
		self.editIcon('button_delete', 'Delete',  button_delete)
		self.editIcon('button_reply', 'Reply',  button_reply)
		self.editIcon('button_move', 'Move',  button_move)
		self.editIcon('button_vote', 'Vote',  button_vote)
		self.editIcon('button_collapse_thread', 'Collapse Thread',  button_collapse_thread)
		self.editIcon('button_expand_thread', 'Expand Thread',  button_expand_thread)
		
		self.editIcon('mime_zip', 'Zip File', mime_zip)        
		self.editIcon('mime_audio', 'Audio File', mime_audio)    
		self.editIcon('mime_video', 'Video File', mime_video)    
		self.editIcon('mime_text', 'Text File', mime_text)      
		self.editIcon('mime_html', 'HTML File', mime_html)      
		self.editIcon('mime_docs', 'Document File', mime_docs)  
		self.editIcon('mime_exe', 'Exe File', mime_exe)        
		self.editIcon('mime_img', 'Image File', mime_img)      
		self.editIcon('mime_unknown', 'Unknown File', mime_unknown)
		

		if REQUEST is not None:
			return self.manage_editForm(self, REQUEST)

	security.declareProtected('Edit NeoBoard', 'manage_batchAction')
	def manage_batchAction(self,
			batch_items=None,
			submit=None,
			to_board=None,
			REQUEST=None):
		"""
			Batch Action
		"""

		if not self.isNeoBoardManager():
			return MessageDialog(title='Not a manager!',
		                             message='You are not authorized to perform this action.',
		                             action=REQUEST['URL1']+'/manage_main'
	                            )						
			

		if batch_items is None or len(batch_items) <= 0:
			return MessageDialog(title='Error!',
		                             message='No item selected!',
		                             action=REQUEST['URL1']+'/manage_main'
	                            )
	                            
		if submit is None or not (submit == 'Batch Delete' or submit == 'Batch Move'):
			return MessageDialog(title='Incorrect usage!',
		                             message='Invalid Submit message.',
		                             action=REQUEST['URL1']+'/manage_main'
	                            )

		if submit == 'Batch Move' and to_board is None:
			return MessageDialog(title='Incorrect usage!',
		                             message='Receiving board is not specified.',
		                             action=REQUEST['URL1']+'/manage_main'
	                            )
		
		if submit == 'Batch Move' and to_board is None:
			return MessageDialog(title='Incorrect usage!',
		                             message='Receiving board is not specified.',
		                             action=REQUEST['URL1']+'/manage_main'
	                            )

		if submit == 'Batch Move' and not self.aq_parent._getOb(to_board).isNeoBoardManager():
			return MessageDialog(title='Not a manager!',
		                             message='You are not a manager for the receiving board.',
		                             action=REQUEST['URL1']+'/manage_main'
	                            )	                            

		if submit == 'Batch Move':
			try:
				receiver = self.aq_parent._getOb(to_board)
			except:
				return MessageDialog(title='No such board!',
			                             message="Receiving board doesn't exist.",
			                             action=REQUEST['URL1']+'/manage_main'
		                            )

    			pointer = self.manage_cutObjects(batch_items)
			receiver.manage_pasteObjects(pointer)
			self._resetNeoBoard()
			receiver._resetNeoBoard()

			REQUEST=setFormMsg(REQUEST, '%d article(s) have been moved to: %s' % (len(batch_items), receiver.title))
		else:
			for article_id in batch_items:
				self._delete_article(article_id)

			self._resetNeoBoard()
			REQUEST=setFormMsg(REQUEST, '%d article(s) have been deleted.' % len(batch_items))
				

		if REQUEST is not None:
			return self.manage_main(REQUEST=REQUEST)
	
	security.declarePrivate('_get_id')		
	def _get_id(self, id):
		new_id = 'a_' + str(self.getNextArticleNum())
		return new_id

	security.declarePrivate('_delete_article')
	def _delete_article(self, id):
		objects = traverseFolder(self._getOb(id), meta_type='NeoBoard Article')
		
		if len(objects) > 1:
			objects.reverse()
		
		for obj in objects:
			obj.aq_inner.aq_parent._delObject(obj.id)
		
		self._delObject(id)
		
	def editManagers(self, managers):
			
		if len(managers) <= 0 or not str(managers[0]).find('||') > 0:
			self.managers = {}
			return
			
		temp_managers = {}
		
		for manager in managers:
			if not str(manager).find('||') > 0:
				return
			username, email = manager.strip().split('||')
			temp_managers[username.strip()] = email.strip()
		
		self.managers = temp_managers
			

	def getManagers(self):
		managers = []
		for username in self.managers.keys():
			managers.append(username + '||' + self.managers[username])
		
		return managers
	
	def getDescription(self):
		return self.description
		
	def isNeoBoardManager(self):
		if self.REQUEST.AUTHENTICATED_USER.has_role('Manager'):
			return 1
			
		return self.REQUEST.AUTHENTICATED_USER.getUserName() in self.managers.keys()
	
	def getNeoBoards(self):
		return self.superValues('NeoBoard')
		
	def getNeoBoardNotices(self):
		return self.objectValues('NeoBoard Notice')
		
	def editPostingCategories(self, categories):
		if len(categories) <= 0 or not str(categories[0]).find('||') > 0:
			self.posting_categories = {}
			return
			
		posting_categories = {}
		
		for category in categories:
			if not str(category).find('||') > 0:
				return
			category_id, category_title = category.strip().split('||')
			posting_categories[category_id.strip()] = category_title.strip()
		
		self.posting_categories = posting_categories
	
	def getPostingCategories(self):
		
		categories = []
		for category_id in self.posting_categories.keys():
			categories.append(category_id + '||' + self.posting_categories[category_id])
		
		return categories
	
	def addIcon(self, id, title):
       		icon_folder = self._getOb('icons')

		file_path = package_home(globals()) + '/www/' + id + '.gif'
		
		addImage(icon_folder, id, title, file_path)
	
	def editIcon(self, id, title, icon):
		if not hasattr(icon, 'filename'):
			return
			
		if icon.filename:
			try:
				self.icons.manage_delObjects([id])
			except:
				pass
	
			image = Image(id, title,'','', '')
			self.icons._setObject(id, image)
			
			self.icons._getOb(id).manage_upload(icon.read())
		else:
			pass			

	def getIcon(self, id, height=None, width=None, alt=None, 
		scale=0, xscale=0, css_class='', hspace=0, vspace=0):
		try:
			icon = getattr(self.icons, id)
			return icon.tag(height=height, width=width, alt=alt, 
				scale=scale, xscale=xscale, css_class=css_class, hspace=hspace, vspace=vspace)			

		except:
			return 'Icon %s Missing' % id
				
	def getCurrentNeoBoard(self):
		return self.aq_chain[0]

	def getNeoBoardURL(self):
		return self.getCurrentNeoBoard().absolute_url()

	def getNeoBoardArticle(self, article):
		if hasattr(self, article):
			return self.article
		return None

	def getNeoBoardArticles(self, skey=None, reverse=0, default=None, default_reverse=0):
		return traverseFolder(self, 'NeoBoard Article', skey=skey, reverse=reverse, default=default, default_reverse=default_reverse)

	def getMaxFileSize(self):
		return self.max_file_size

	def getMaxImageWidth(self):
		return self.max_image_width
	
	def getMaxImageHeight(self):
		return self.max_image_height

	def checkSize(self, file):
		filesize = int(len(file.read())/1047576)
		file.seek(0)
		if filesize > self.getMaxFileSize():
			return 0
		else:
			return 1

	def getMaxListTitleLength(self):
		return self.max_list_title_length

	def getNextArticleNum(self, notice=0):
		new_num = self.next_article_num + 1
		
		if notice:
			num_notices = self.objectValues('NeoBoard Notice')
			return len(num_notices) + 1
		
		if hasattr(self, 'a_' + str(new_num)):
			self._resetNeoBoard()
			new_num = self.next_article_num + 1
		
		return new_num

	def setNextArticleNum(self, num):
		self.next_article_num = num
	
	def setTotalNumArticles(self, num):
		self.total_num_articles = num 

	def getTotalNumArticles(self):
		return self.total_num_articles

	def getCurrentArticleURL(self):
		url = self.REQUEST.URL1
		return url.replace(self.getNeoBoardURL() + '/', '')

	def getArchivePath(self):
		return self.archive_path

	def isMailEnabled(self):
		return self.mail_article		
	
	def isReadOnly(self):
		return self.read_only

	def pageCeil(self, num1, num2):
		
		quotient, remainder = divmod(int(num1), int(num2))
		
		if remainder > 0:
			return quotient + 1
		else:
			return quotient	
			
	def sendmail(self, from_addr, to_addr, reply_addr, subject, body, attachments=None, html=0):		
		mailer = NB_Mailer(from_addr, to_addr, reply_addr, subject, body, attachments, html)
		mailer.send()
					
	
	def index_html(self, REQUEST={}, RESPONSE=None):
		"""
			Own index_html
		"""

		header = apply(self.neoboard_header.__call__, (self, REQUEST, RESPONSE))
	        footer = apply(self.neoboard_footer.__call__, (self, REQUEST, RESPONSE))

      		return header + footer

	manage_main = index_html

	def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):      			
		return self.index_html()
		
	def _resetNeoBoard(self):
		articles =[]
		for art in self.objectValues('NeoBoard Article'):
			articles.append(art.id)
			
		num = len(articles)
			
		next_id = getNextElementName(articles, prefix='')

		if num > 0:	
			self.setNextArticleNum(int(next_id) - 1)
			self.setTotalNumArticles(num)
		else:
			self.setNextArticleNum(0)
			self.setTotalNumArticles(0)

	security.declareProtected('Edit NeoBoard', 'selfUpgrade')
	def selfUpgrade(self):
		"""
		"""
		if not hasattr(self, 'read_only'):
			self.read_only=0
  	
InitializeClass(NeoBoard)